/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright (c) 1997-2018 Oracle and/or its affiliates. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common Development
 * and Distribution License("CDDL") (collectively, the "License").  You
 * may not use this file except in compliance with the License.  You can
 * obtain a copy of the License at
 * https://oss.oracle.com/licenses/CDDL+GPL-1.1
 * or LICENSE.txt.  See the License for the specific
 * language governing permissions and limitations under the License.
 *
 * When distributing the software, include this License Header Notice in each
 * file and include the License file at LICENSE.txt.
 *
 * GPL Classpath Exception:
 * Oracle designates this particular file as subject to the "Classpath"
 * exception as provided by Oracle in the GPL Version 2 section of the License
 * file that accompanied this code.
 *
 * Modifications:
 * If applicable, add the following below the License Header, with the fields
 * enclosed by brackets [] replaced by your own identifying information:
 * "Portions Copyright [year] [name of copyright owner]"
 *
 * Contributor(s):
 * If you wish your version of this file to be governed by only the CDDL or
 * only the GPL Version 2, indicate your decision by adding "[Contributor]
 * elects to include this software in this distribution under the [CDDL or GPL
 * Version 2] license."  If you don't indicate a single choice of license, a
 * recipient has the option to distribute your version of this file under
 * either the CDDL, the GPL Version 2 or to extend the choice of license to
 * its licensees as provided above.  However, if you add GPL Version 2 code
 * and therefore, elected the GPL Version 2 license, then the option applies
 * only if the new code is made subject to such option by the copyright
 * holder.
 */

package org.glassfish.pfl.dynamic.codegen.impl;

import java.util.Map ;
import java.util.LinkedHashMap ;

import org.glassfish.pfl.dynamic.codegen.spi.Type ;
import org.glassfish.pfl.dynamic.codegen.spi.Variable ;

import org.glassfish.pfl.dynamic.codegen.impl.StatementBase ;
import org.glassfish.pfl.basic.contain.Pair;

/**
 *
 * @author Ken Cavanaugh
 */
public final class TryStatement extends StatementBase {
    private BlockStatement bodyPart ;
    private BlockStatement finalPart ;
    private Map<Type,Pair<Variable,BlockStatement>> catches ;

    TryStatement( Node parent ) {
	super( parent ) ;
	bodyPart = new BlockStatement( this ) ;
	finalPart = new BlockStatement( this ) ;
	catches = new LinkedHashMap<Type,Pair<Variable,BlockStatement>>() ;
    }

    public BlockStatement bodyPart() {
	return this.bodyPart ;
    }
    
    public BlockStatement finalPart() {
	return this.finalPart ;
    }
    
    public Map<Type,Pair<Variable,BlockStatement>> catches() {
	return this.catches ;
    }

    /** Add a new Catch block to this try statement.  type must be
     * a non-primitive, non-array type, and may occur only once per
     * try statement.
     */
    public Pair<Variable,BlockStatement> addCatch( Type type, String ident ) {
	if (type.isPrimitive())
	    throw new IllegalArgumentException( "Primitive type " + type +
		" not allowed in catch block" ) ;

	if (type.isArray())
	    throw new IllegalArgumentException( "Array type " + type +
		" not allowed in catch block" ) ;

	if (catches.containsKey( type )) 
	    throw new IllegalArgumentException( "Type " + type + 
		" is already used as a catch block in this try statement" ) ;

	// XXX iterate over list and make sure that type is not a subclass of
	// any previous types in the list
	
	BlockStatement stmt = new BlockStatement( this ) ;
	Variable var = stmt.exprFactory().variable( type, ident ) ;
	Pair<Variable,BlockStatement> result = new Pair<Variable,BlockStatement>(
	    var, stmt ) ;
	catches.put( type, result ) ;
	return result ;
    }
	
    public void accept( Visitor visitor ) {
	visitor.visitTryStatement( this ) ;
    }
}
